@@ -1,5 +1,5 @@
 __author__ = 'Antony Lewis'
-__version__ = "1.2.1"
+__version__ = "1.2.2"
 __url__ = "https://getdist.readthedocs.io"
 
 from getdist.inifile import IniFile
@@ -12,8 +12,8 @@ import sys
 if sys.version_info < (3, 7):
     import platform
 
-    if platform.python_implementation() != 'CPython':
-        raise ValueError('Only CPython is supported on Python 3.6')
+    if platform.python_implementation() not in ['CPython', 'PyPy']:
+        raise ValueError('Only CPython and PyPy is supported on Python 3.6')
 
 
 def get_defaults_file(name='analysis_defaults.ini'):
