@@ -1,5 +1,4 @@
 import os
-import random
 import numpy as np
 import re
 from getdist.paramnames import ParamNames, ParamInfo, escapeLatex
@@ -9,7 +8,7 @@ import pickle
 import logging
 from copy import deepcopy
 from collections import namedtuple
-from typing import Sequence, Any, Optional, Union
+from typing import Sequence, Any, Optional, Union, List
 
 # whether to write to terminal chain names and burn in details when loaded from file
 print_load_details = True
@@ -27,14 +26,12 @@ class WeightedSampleError(Exception):
     """
     An exception that is raised when a WeightedSamples error occurs
     """
-    pass
 
 
 class ParamError(WeightedSampleError):
     """
     An Exception that indicates a bad parameter.
     """
-    pass
 
 
 def last_modified(files):
@@ -44,7 +41,7 @@ def last_modified(files):
     :param files: An iterable of file names.
     :return: The latest "last modified" time
     """
-    return max([os.path.getmtime(fname) for fname in files if os.path.exists(fname)])
+    return max(os.path.getmtime(fname) for fname in files if os.path.exists(fname))
 
 
 def slice_or_none(x, start=None, end=None):
@@ -898,22 +895,24 @@ class WeightedSamples:
 
         return thin_ix
 
-    def random_single_samples_indices(self):
+    def random_single_samples_indices(self, random_state=None):
         """
         Returns an array of sample indices that give a list of weight-one samples, by randomly
         selecting samples depending on the sample weights
 
+        :param random_state: random seed or Generator
         :return: array of sample indices
         """
+        random_state = np.random.default_rng(random_state)
         max_weight = np.max(self.weights)
         thin_ix = []
         for i in range(self.numrows):
             P = self.weights[i] / max_weight
-            if random.random() < P:
+            if random_state.random() < P:
                 thin_ix.append(i)
         return np.array(thin_ix, dtype=int)
 
-    def thin(self, factor):
+    def thin(self, factor: int):
         """
         Thin the samples by the given factor, giving set of samples with unit weight
 
@@ -923,21 +922,16 @@ class WeightedSamples:
         self.setSamples(self.samples[thin_ix, :], loglikes=None if self.loglikes is None else self.loglikes[thin_ix],
                         min_weight_ratio=-1)
 
-    def weighted_thin(self, factor):
+    def weighted_thin(self, factor: int):
         """
-        Thin the samples by the given factor, preserving the weights.
-        This function also preserves separate chains.
+        Thin the samples by the given factor, preserving the weights (not all set to 1).
+
         :param factor: The (integer) factor to thin by
         """
         unique, counts = self.thin_indices_and_weights(factor, self.weights)
         self.setSamples(self.samples[unique, :],
-                        loglikes=None if self.loglikes is None
-                        else self.loglikes[unique],
-                        weights=counts,
-                        min_weight_ratio=-1)
-        if self.chain_offsets is not None:
-            self.chain_offsets = np.array([np.sum(unique < off)
-                                           for off in self.chain_offsets])
+                        loglikes=None if self.loglikes is None else self.loglikes[unique],
+                        weights=counts, min_weight_ratio=-1)
 
     def filter(self, where):
         """
@@ -958,6 +952,7 @@ class WeightedSamples:
         scale = np.min(logLikes)
         if self.loglikes is not None:
             self.loglikes += logLikes
+        self.weights = np.asarray(self.weights, dtype=np.float64)
         self.weights *= np.exp(-(logLikes - scale))
         self._weightsChanged()
 
@@ -969,7 +964,8 @@ class WeightedSamples:
         """
         MaxL = np.max(self.loglikes)
         newL = self.loglikes * cool
-        self.weights = self.weights * np.exp(-(newL - self.loglikes) - (MaxL * (1 - cool)))
+        self.weights = np.asarray(self.weights, dtype=np.float64)
+        self.weights *= np.exp(-(newL - self.loglikes) - (MaxL * (1 - cool)))
         self.loglikes = newL
         self._weightsChanged()
 
@@ -1047,6 +1043,9 @@ class WeightedSamples:
                    np.hstack((self.weights.reshape(-1, 1), loglikes.reshape(-1, 1), self.samples)),
                    fmt=self.precision)
 
+    def __getitem__(self, item):
+        return self._makeParamvec(item)
+
 
 # noinspection PyAttributeOutsideInit
 class Chains(WeightedSamples):
@@ -1074,6 +1073,7 @@ class Chains(WeightedSamples):
         """
 
         self.chains = None
+        self.chain_offsets = None
         super().__init__(**kwargs)
         self.jobItem = jobItem
         self.ignore_lines = float(kwargs.get('ignore_rows', 0))
@@ -1128,7 +1128,7 @@ class Chains(WeightedSamples):
         """
 
         if self.chains is None:
-            if hasattr(self, 'chain_offsets'):
+            if self.chain_offsets is not None:
                 # must update chain_offsets to be able to correctly split back into separate filtered chains if needed
                 lens = [0]
                 for off1, off2 in zip(self.chain_offsets[:-1], self.chain_offsets[1:]):
@@ -1138,6 +1138,24 @@ class Chains(WeightedSamples):
         else:
             raise ValueError('chains are separated, makeSingle first or call filter on individual chains')
 
+    def weighted_thin(self, factor: int):
+        """
+        Thin the samples by the given factor, giving (in general) non-unit integer weights.
+        This function also preserves separate chains.
+
+        :param factor: The (integer) factor to thin by
+        """
+        if not self.chains and self.chain_offsets is None:
+            return super().weighted_thin(factor)
+        has_chains = self.chains
+        chains = self.getSeparateChains()
+        for chain in chains:
+            chain.weighted_thin(factor)
+        self.chains = chains
+        if not has_chains:
+            self.makeSingle()
+        self.needs_update = True
+
     def getParamNames(self):
         """
         Get :class:`~.paramnames.ParamNames` object with names for the parameters
@@ -1251,7 +1269,15 @@ class Chains(WeightedSamples):
         if isinstance(par, ParamInfo):
             par = par.name
         if isinstance(par, str):
-            return self.samples[:, self.index[par]]
+            index = self.index.get(par)
+            if index is not None:
+                return self.samples[:, index]
+            if par == 'weight':
+                return self.weights
+            elif par == 'loglike':
+                return self.loglikes
+            else:
+                raise ParamError('Unknown parameter %s' % par)
         return super()._makeParamvec(par)
 
     def updateChainBaseStatistics(self):
@@ -1408,6 +1434,8 @@ class Chains(WeightedSamples):
 
         :return: self
         """
+        if not self.chains:
+            raise ValueError('There are no separated chains for makeSingle()')
         self.chain_offsets = np.cumsum(np.array([0] + [chain.samples.shape[0] for chain in self.chains]))
         weights = None if self.chains[0].weights is None else np.hstack([chain.weights for chain in self.chains])
         loglikes = None if self.chains[0].loglikes is None else np.hstack([chain.loglikes for chain in self.chains])
@@ -1416,7 +1444,7 @@ class Chains(WeightedSamples):
         self.needs_update = True
         return self
 
-    def getSeparateChains(self):
+    def getSeparateChains(self) -> List['WeightedSamples']:
         """
         Gets a list of samples for separate chains.
         If the chains have already been combined, uses the stored sample offsets to reconstruct the array
@@ -1427,9 +1455,12 @@ class Chains(WeightedSamples):
         if self.chains is not None:
             return self.chains
         chainlist = []
-        for off1, off2 in zip(self.chain_offsets[:-1], self.chain_offsets[1:]):
-            chainlist.append(WeightedSamples(samples=self.samples[off1:off2], weights=self.weights[off1:off2],
-                                             loglikes=self.loglikes[off1:off2]))
+        if self.chain_offsets is None:
+            raise WeightedSampleError('Samples were not combined from separate chains')
+        else:
+            for off1, off2 in zip(self.chain_offsets[:-1], self.chain_offsets[1:]):
+                chainlist.append(WeightedSamples(samples=self.samples[off1:off2], weights=self.weights[off1:off2],
+                                                 loglikes=self.loglikes[off1:off2]))
         return chainlist
 
     def removeBurnFraction(self, ignore_frac):
